# -*- coding: utf-8 -*-
"""
Created on Thu May 23 00:37:17 2019

@author: MAndrle
"""

import sys

# load necessities...
import pandas as pd
import numpy as np

# graphics
import matplotlib.pyplot as plt
import seaborn as sns 
sns.set_style("white") # i don't like the gray stuff...


# import "IMF utilities"
sys.path.append("C:/Users/MAndrle/Box Sync/michal/IMF/IMF_ML_COURSE_2017/code_python/michal")
import utilities as u

# prepare for Random Forests
from sklearn.ensemble import RandomForestClassifier
from sklearn.datasets import make_classification

# prepare for logistic regression
from sklearn.linear_model import LogisticRegression

# we'll need this for evaluation
from sklearn.metrics import roc_curve, auc


## Load the 'Breast Cancer Data' [directly from sci-kit learn]
""" Data description:
    
   
1. variance of Wavelet Transformed image (continuous) 
2. skewness of Wavelet Transformed image (continuous) 
3. curtosis of Wavelet Transformed image (continuous) 
4. entropy of image (continuous) 
5. class (integer)  
   
The data is sorted by the CLASS [5].
    
"""

# read the csv data
data = pd.read_csv('C:/Users/MAndrle/Box Sync/michal/IMF/IMF_ML_COURSE_2017/dataset_examples/banknotes2012/data_banknote_authentication.txt')


# do some HEAVY feature engineering... [not really this time]
# Should i adjust the skew of some features?

# add names to the columns
data.columns = ['variance','skewness','curtosis','entropy','class']


def train_test(X, Y):
    train_portion = 0.7  
    
    N = len(Y)
    
    all_ix   = np.arange(N)
    ix_train = np.random.choice(N,np.int(train_portion * N), replace=False) # resample indices
    ix_train.sort() # just for the fun of it...
    ix_test  = np.setdiff1d(all_ix, ix_train)  # set difference, to find 'test'
    
    X_train = X[ix_train, ]
    X_test  = X[ix_test,  ]
    
    Y_train = Y[ix_train, ]
    Y_test  = Y[ix_test,  ]
    
    return(Y_train, X_train, Y_test, X_test)


	
# Now, since this data is simple lets convert to NDARRAY right away
X = data.values[:,0:-1]
Y = data.values[:,-1]   


# split the sample into Training and Testing
(Y_tr, X_tr, Y_tst, X_tst) = train_test(X, Y)



## From now on, we're gonna look only at TRAINING dataset...
data.describe()


# convert the data into a "long form", se we can do 'violins' for features...
melt_data = pd.melt(data,id_vars="class",
                    var_name="Features",
                    value_name='Value')

plt.figure(figsize=(13,10))
sns.violinplot(x="Features", y="Value", hue="class", data=melt_data, split=True)

#plt.figure(figsize=(13,10))
#sns.swarmplot(x='Features', y='Value', hue='class', data=melt_data)



## ------------------------------------------------------------------------
## Train a Logistic Regression
## ------------------------------------------------------------------------
lrc = LogisticRegression(random_state=0, solver='lbfgs')
lrc.fit(X_tr, Y_tr)

pl_tr = lrc.predict_proba(X_tst)
Y_tst_lr = lrc.predict(X_tst)

print("Logistic Regression \n")
u.get_metrics(Y_tst, Y_tst_lr)



## ------------------------------------------------------------------------
## Train a Random Forest
## ------------------------------------------------------------------------
# should we tune this...?
maxdepth = 4

rfc = RandomForestClassifier(n_estimators=50, criterion='gini', 
                             max_depth=maxdepth, random_state=0)

#X_tr = X_tr[:,0:4]

rfc.fit(X_tr, Y_tr)


# probabilistic forecast
p_tr  = rfc.predict_proba(X_tr)
p_tst = rfc.predict_proba(X_tst)

# classification with a default treshold of 0.5 [now, that may be quite bad]
Y_tst_hat = rfc.predict(X_tst)

## ------------------------------------------------------------------------
## Key metrics to check...
## ------------------------------------------------------------------------
info1 = u.get_metrics(Y_tst, Y_tst_hat)

#plt.plot(p_tr[:,1])
#plt.plot(Y_tr)


## ------------------------------------------------------------------------
## Take a look at the ROC
## ------------------------------------------------------------------------


# compute the axis and thresholds
(fpr, tpr, thresholds)    = roc_curve(Y_tr, p_tr[:, 1])
fts, tts, thresholds_ts = roc_curve(Y_tst, p_tst[:, 1])

# Compute also the "Area Under the Curve" (AUC)
roc_auc_tr = auc(fpr, tpr)
roc_auc_ts = auc(fts, tts)

# do the chart
plt.figure(figsize=(9,7))
plt.plot([0, 1], [0, 1], linestyle='--', lw=2, color='r', label='Pure Chance', alpha=.8)

plt.plot(fpr, tpr, color='b', lw=2, alpha=.8, label='Training Sample')
plt.plot(fts, tts, color='y', lw=2, alpha=.8, label='Test Sample')

plt.title('ROC Curve for Bank Note Data (Test AUC = %2.3f)' % (roc_auc_ts))

plt.xlim([-0.05, 1.05])
plt.ylim([-0.05, 1.05])
plt.xlabel('False Positive Rate')
plt.ylabel('True Positive Rate')

plt.legend(loc="lower right")
plt.show()

## Confusion matrix a la Matlab [IMF]
f_, ax_ = u.create_confusion_matrix_plot(Y_tst, Y_tst_hat, ['1','0'], title='Bank Notes')



## A quick look at feature importance
plt.figure(figsize=(10,7))
plt.bar(np.arange(4), rfc.feature_importances_)
plt.xticks(np.arange(4), data.columns[0:5], fontsize=12)

print('The order of importance should not be surprising in this small dataset...')

























































